<?php
namespace Drupal\dbee\Tests;

use Drupal\user\Entity\User;
use Drupal\aes\AES;

/**
 * enable/uninstall dbee module, Encrypting/Decrypting all existing users.
 *
 * When the dbee module is enabled or disabled or when the AES encryption key changes.
 *
 * @group dbee
 */
class DbeeAllUsersTestCase extends DbeeWebTestBase {

  /**
   * User with permission to enable/uninstall modules.
   *
   * @var array
   */
  protected $admin_modules_account;

  /**
   * User with permission change AES settings.
   *
   * @var array
   */
  protected $admin_aes_account;

  /**
   * Nomber of basic user to create.
   *
   * @var array
   */
  protected $n_users = 10;

  /**
   * Array keyed by uid providing users mail and int original values.
   *
   * @var array
   */
  protected $users_info = array();

  /**
   * Number of users stored in db.
   *
   * @var array
   */
  protected $total_users = 0;

  /**
   * Number of users stored in db taht should be encrypted (no empty mail and init).
   *
   * @var array
   */
  protected $n_updated_users = 0;


  /**
   * Set up function
   */
  public function setUp() {
    // Enable any modules required for the test.
    parent::setUp();
    // Create a user who can edit the aes options.
    $this->admin_aes_account = $this->drupalCreateUser(array('administer aes'));
    // Create a user who can enable the dbee module.
    $this->admin_modules_account = $this->drupalCreateUser(array('administer modules'));

    $this->dbee_create_many_users();
  }


  /**
   * Enabled and unnistall the dbee module, checking if encryption/decryption for
   * existing users.
   */
  public function testAllUsers() {

    // The dbee module is not enable yet. Test users emails.
    $this->assertTrue($this->dbee_all_users_valid($this->users_info, FALSE), 'All users are uncrypted');

    // Enable the dbee module.
    $this->drupalLogin($this->admin_modules_account);
    $edit0 = array('modules[Other][dbee][enable]' => TRUE);
    $this->drupalPostForm('admin/modules', $edit0, t('Install'));
    $this->rebuildContainer();
    $this->assertTrue($this->container->get('module_handler')->moduleExists('dbee'), 'dbee module is enabled now');
    $message = dbee_update_crypt_users_all_message('encrypt', $this->total_users, $this->n_updated_users);
    $this->assertRaw($message, $message);
    // Test all email address.
    $this->assertTrue($this->dbee_all_users_valid($this->users_info), 'All users are correctly stored');


    // Uninstall the dbee module.
    $edit1 = array('uninstall[dbee]' => TRUE);
    $this->drupalPostForm('admin/modules/uninstall', $edit1, t('Uninstall'));
    // Confirmation page.
    $this->drupalPostForm('admin/modules/uninstall/confirm', array(), t('Uninstall'));
    $this->rebuildContainer();
    $this->assertFalse($this->container->get('module_handler')->moduleExists('dbee'), 'dbee module is uninstalled now');
    $message = dbee_update_crypt_users_all_message('decrypt', $this->total_users, $this->n_updated_users);
    $this->assertRaw($message, $message);    // Test all email address.
    $this->assertTrue($this->dbee_all_users_valid($this->users_info, FALSE), 'All users are uncrypted and valid');


    // Enable again dbee module (not the same that install module).
    $edit2 = array('modules[Other][dbee][enable]' => 'dbee');
    $this->drupalPostForm('admin/modules', $edit2, t('Install'));
    $this->rebuildContainer();
    $this->assertTrue($this->container->get('module_handler')->moduleExists('dbee'), 'dbee module is enabled again');
    $message = dbee_update_crypt_users_all_message('encrypt', $this->total_users, $this->n_updated_users);
    $this->assertRaw($message, $message);    // Test all email address.
    $this->assertTrue($this->dbee_all_users_valid($this->users_info), 'All users are correctly stored');

    // Edit the AES encryption key.
    //$this->privileged_user = $this->drupalCreateUser( array('administer aes'));
    $this->drupalLogin($this->admin_aes_account, FALSE);
    $new_key = AES::make_key();
    $edit3 = array(
      'key' => $new_key,
      'key_confirm' => $new_key,
    );
    $this->drupalPostForm('admin/config/system/aes', $edit3, t('Save'));
    // Test all email address.
    // Compare them and set decrypt and encrypt parameters.
    $message = dbee_update_crypt_users_all_message('change', $this->total_users, $this->n_updated_users);
    $this->assertRaw($message, $message);
    $this->assertTrue($this->dbee_all_users_valid($this->users_info, TRUE), 'All users are correctly stored');
  }

  /**
   * Create many users with warious values of mail and init properties.
   *
   * Create as $this->n_users users. it recommanded to create at least 10 users
   * to have all possibles kind of values of mail and init. In order to create
   * thoses users, simply set the $this->n_users into the class that extends the
   * DbeeWebTestBase class and call this method from the setUp. This function
   * will set $this->users_info, $this->total_users, $this->n_updated_users,
   */
  protected function dbee_create_many_users() {
    // First create 10 users with variant email addresses : sensitive case,
    // lowercase, empty and invalid.
    $all_datas = array();
    $provider = '@eXample.com';
    for ($i = 1; $i <= $this->n_users; $i++) {
      // Create users, sensitive case and lower case.
      $edit = array();
      if (isset($account)) {
        unset($account);
      }
      $edit['name'] = $this->randomMachineName();
      $edit['mail'] = $edit['name'] . $provider;

      // 3 sensitive case email addresses (5 - 2).
      if ( ceil($i / 2) == $i / 2 ) {
        // 4 lowercase email addresses (5-1).
        $edit['mail'] = \Drupal\Component\Utility\Unicode::strtolower($edit['mail']);
      }

      if ( ceil($i / 9) == $i / 9 ) {
        // 1 empty email addresses.
        $edit['mail'] = '';
      }
      if ( ceil($i / 5) == $i / 5 ) {
        // 2 invalid email address.
        $edit['mail'] = 'This-is-an-invalid-email';
        // The second one is lower case.
        if ($i > 5) {
          $edit['mail'] = \Drupal\Component\Utility\Unicode::strtolower($edit['mail']);
        }
      }

      $edit['pass'] = user_password();
      $edit['status'] = 1;
      $edit['init'] = $edit['mail'];

      $account = User::create($edit);
      $account->save();

      $this->assertTrue($account->id(), t('User creation succedd for email @email', array('@email' => $edit['mail'])));

      // Store datas for checking.
      if (!empty($account->id())) {
        $this->users_info[$account->id()] = array(
          'mail' => $edit['mail'],
          'init' => $edit['init'],
        );
      }
    }
    $this->assertTrue(count($this->users_info) == $this->n_users, t('@user users have been created successfully', array('@user' => $this->n_users)));

    // Count users
    $no_update_user = 0;
    $this->total_users = 0;
    $query_users = db_query("SELECT mail, init FROM {users_field_data} WHERE uid <> :uid", array(':uid' => 0));
    while ($row_users = $query_users->fetchAssoc()) {
      $this->total_users++;
      $update = FALSE;
      foreach (array('mail', 'init') as $field) {
        if (!empty($row_users[$field]) && \Drupal::service('email.validator')->isValid($row_users[$field])) {
          $update = TRUE;
          break;
        }
      }
      if (!$update) {
        $no_update_user++;
      }
    }
    $this->n_updated_users = $this->total_users - $no_update_user;
  }
}
