<?php
namespace Drupal\dbee\Tests;

use Drupal\user\Entity\User;

/**
 * Add/register users.
 *
 * Test the user registration, 'add a new user' from the admin interface, create
 * new user programmatically, make sure the system do not validate duplicate
 * emails.
 *
 * @group dbee
 */
class DbeeNewUsersTestCase extends DbeeWebTestBase {
  protected $admin_users_account;
  protected $existing_user;
  protected $to_delete_account;

  public static $modules = array('dbee');

  public function setUp() {
    // Enable any modules required for the test.
    parent::setUp();

    // Create a basic user with mail = 'example@example.com'. This email will
    // be used to testing if the system prevent from creating a new user with
    // an existing email.
    // Create a user, with sensitive case mail.
    $this->existing_user = $this->drupalCreateUser();
    // Reload the existing user to encrypt the email address.
    // To reset the user cache, use EntityStorageInterface::resetCache().
    $this->existing_user = \Drupal::entityManager()->getStorage('user')->load($this->existing_user->id());

    $this->admin_users_account = $this->drupalCreateUser(array(
      'administer users',
      'access user profiles',
    ));
  }

  public function testNewUsers() {
    // Create new user form register page, then this new user try to edit it,
    // test sensitive case feature, test for dooblon.

    // Try to create a new user from  the register form.
    $name = $this->randomMachineName();
    $mail = $name . '@exAMple.com';
    $edit1 = array(
      'name' => $name,
      'mail' => $mail,
    );
    $this->drupalPostForm('user/register', $edit1, t('Create new account'));
    $this->assertRaw(t('A welcome message with further instructions has been sent to your email address.'), 'Anonymous user successfully registered.');
    $register_uid = db_query('SELECT uid FROM {users_field_data} WHERE name = :name', array(':name' => $name))->fetchField();
    // Test the email address.
    $data_register = array(
      $register_uid => array(
        'mail' => $mail,
        'init' => $mail,
      ),
    );
    $this->assertTrue($this->dbee_all_users_valid($data_register), 'The new user from the register page is correctly encrypted.');

    // Try to create a new user with an email already register, the system
    // should not validate it.
    $password  = user_password();
    $edit2 = array(
      'name' => $this->randomMachineName(),
      // Set email to 'STRING@EXAMPLE.COM'.
      'mail' => \Drupal\Component\Utility\Unicode::strtoupper($this->existing_user->mail->value),
    );
    $this->drupalPostForm('user/register', $edit2, t('Create new account'));
    $this->assertRaw(t('The email address %email is already taken', array('%email' => $edit2['mail'])), 'The module successfully detects when someone trying to register the same email twice, even if there is case conflict.');

    // Try to create a new user from the admin interface.
    $this->drupalLogin($this->admin_users_account);
    $password3 = user_password();
    $name3 = $this->randomMachineName();
    $mail3 = $name3 . '@example.com';
    $edit3 = array(
      'name' => $name3,
      'mail' => $mail3,
      'pass[pass1]' => $password3,
      'pass[pass2]' => $password3,
    );
    $this->drupalPostForm('admin/people/create', $edit3, t('Create new account'));
    // Check if new user account has not been created.
    // From the user_profile_form_submit() function.
    $added_uid = db_query('SELECT uid FROM {users_field_data} WHERE name = :name', array(':name' => $name3))->fetchField();
    $this->assertRaw(t('Created a new user account for <a href=":url">%name</a>. No email has been sent.', array(':url' => \Drupal\Core\Url::fromRoute('entity.user.canonical', ['user' => $added_uid])->toString(), '%name' => $name3)), 'New user successfully added from the admin interface.');
    // Test the email address.
    $data_added = array(
      $added_uid => array(
        'mail' => $mail3,
        'init' => $mail3,
      ),
    );
    $this->assertTrue($this->dbee_all_users_valid($data_added), 'The new user from the admin interface is correctly encrypted');

    // Try to create a new user with an email already register, the system
    // should not validate it.
    $password4 = user_password();
    $name4 = $this->randomMachineName();
    // Set email to 'STRING@EXAMPLE.COM'.
    $mail4 = \Drupal\Component\Utility\Unicode::strtoupper($this->existing_user->mail->value);
    $edit4 = array(
      'name' => $name4,
      'mail' => $mail4,
      'pass[pass1]' => $password4,
      'pass[pass2]' => $password4,
    );
    $this->drupalPostForm('admin/people/create', $edit4, t('Create new account'));
    // Check if new user account has not been created.
    $this->assertRaw(t('The email address %email is already taken.', array('%email' => $mail4)), 'Trying to add a new user with an existing email displays an error message, even if there is case conflict');

    // Attempt to bypass duplicate email registration validation by adding spaces.
    $edit['mail'] = '   ' . $mail4 . '   ';

    $this->drupalPostForm('admin/people/create', $edit, t('Create new account'));
    $this->assertText(t('The email address @email is already taken.', array('@email' => $mail4)), 'Supplying a duplicate email address with added whitespace displays an error message');


    // Try to save a new user programatically.
    $password5 = user_password();
    $name5 = $this->randomMachineName();
    $mail5 = $name5 . '@exAMple.com';
    $prog_user_array = array(
      'name' => $name5,
      'pass' => $password5, // note: do not md5 the password
      'mail' => $mail5,
      'status' => 1,
      'init' => $mail5,
    );
    $prog_user = User::create($prog_user_array);
    $prog_user->save();

    // Test the email address.
    $data_prog = array(
      $prog_user->id() => $prog_user_array,
    );
    $this->assertTrue($this->dbee_all_users_valid($data_prog), 'The new user from php is correctly encrypted');
    $this->assertEqual($prog_user->mail->value, $mail5, 'The mail value returned by the user_save() fonction is valid (decrypted)');
    $this->assertEqual($prog_user->init->value, $mail5, 'The init value returned by the user_save() fonction is valid (decrypted)');
  }
}
