<?php
namespace Drupal\dbee\Tests;

use Drupal\simpletest\WebTestBase;

/**
 * Base class for the dbee modules tests.
 *
 * Correctly install the aes module, provide email variables and a testing function.
 *
 * NO @_group dbee : don't want this class to be discovered.
 */
class DbeeWebTestBase extends WebTestBase {

  /**
  * aes.settings is not completly set, see issue #2805325.
  *
  * @var boolean
  */
  protected $strictConfigSchema = FALSE;

  /**
   * Modules to enable Do not enable the dbee module yet.
   *
   * @var array
   */
  public static $modules = array('user', 'aes');

  /**
   * Simple Lowercase email adress.
   *
   * @var string
   */
  protected $lowercase_email = 'example@example.com';

  /**
   * Sensitive case Lowercase email adress.
   *
   * @var string
   */
  protected $sensitivecase_email = 'ExAMpLe@ExAMpLe.com';

  /**
   * Invalid email adress.
   *
   * @var string
   */
  protected $invalid_email = 'exaMple!@example,com';

  /**
   * Empty email adress.
   *
   * @var string
   */
  protected $empty_email = '';

  /**
   * Aes module needs to set the active directory in the setting.php file in
   * order to be installed. Set it here.
   *
   * @var string
   */
  protected function writeSettings(array $settings) {
    $settings['config_directories'][CONFIG_ACTIVE_DIRECTORY] = (object) array(
      'value' => $this->privateFilesDirectory . '/aes',
      'required' => TRUE,
    );
    parent::writeSettings($settings);
  }

  /**
   * This class could be loaded even if the dbee module is not enabled. Make
   * sure that the dbee functions are availables.
   */
  public function setUp() {
    parent::setUp();
    if (!function_exists('dbee_encrypt')) {
      module_load_include('module', 'dbee');
    }
  }

  /**
   * Check that the emails stored in db are the expected ones.
   *
   * Check the 4 values : mail, init, and the sensitive case mail (dbee). Try
   * to decrypt it back.
   *
   * @param array $users_info
   *   Multidimensional array storing user mail and init original values
   *  (uncrypted) keyed by the user ID.
   * @param boolean $installed
   *   Inform if the dbee module is enabled. TRUE for enabled (datas should be
   *   encrypted, FALSE for disabled (datas should be decrypted). Default is
   *   TRUE.
   */
  protected function dbee_all_users_valid($users_info, $installed = TRUE) {
    // Test all email address.
    $all_succeed = TRUE;
    foreach ($users_info as $uid => $datas) {
      $mail_is_invalid = (empty($datas['mail']) || !\Drupal::service('email.validator')->isValid($datas['mail']));
      $init_is_invalid = (empty($datas['init']) || !\Drupal::service('email.validator')->isValid($datas['init']));

      // Set expected values for that user.
      $expected = array(
        'mail' => (!$installed || $mail_is_invalid) ? $datas['mail'] : dbee_encrypt($datas['mail'], FALSE),
        'init' => (!$installed || $init_is_invalid) ? $datas['init'] : dbee_encrypt($datas['init'], FALSE),
        'dbee_lc_mail' =>(!$installed) ? NULL : (($mail_is_invalid) ? \Drupal\Component\Utility\Unicode::strtolower($datas['mail']) : dbee_encrypt($datas['mail'], TRUE)),
        'dbee_lc_init' => (!$installed) ? NULL : (($init_is_invalid) ? \Drupal\Component\Utility\Unicode::strtolower($datas['init']) : dbee_encrypt($datas['init'], TRUE)),
      );

      $default = array(
        'mail' => NULL,
        'init' => NULL,
        'dbee_lc_mail' => NULL,
        'dbee_lc_init' => NULL,
      );
      // Retreive current values for that user.
      $stored = db_query('SELECT mail, init FROM {users_field_data} WHERE uid = :uid', array(':uid' => $uid))->fetchAssoc();
      if ($installed) {
        $stored_lc = dbee_retreive_db_lc_values($uid);
        if (is_array($stored_lc)) {
          $stored += $stored_lc;
        }
      }
      $stored += $default;

      // Compare values.
      $compare = array_diff_assoc($expected, $stored);

      if ($compare != array()) {
        $all_succeed = FALSE;
        foreach ($compare as $field => $value) {
          $arg = array(
            '%field' => $field,
            '%stored_value' => $stored[$field],
            '%expected_value' => $expected[$field],
          );
          $this->assertTrue(FALSE, t('the stored %field (%stored_value) is not the expected one (%expected_value)', $arg));
        }
      }
    }
    return $all_succeed;
  }
}
